<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method='text' indent="no"/>

<!-- GETTING AUTOMATA NAME FROM ANY CHILD -->
<xsl:template name="automataName"><xsl:value-of select="ancestor-or-self::stateMachine/@name" /></xsl:template>

<!-- MISSED AUTOMATAS -->
<xsl:variable name="missedAutomatas" 
    select="//actionNode[@type='AUTOMATA_CALL']
    [not(parameter[@name='automata']/@value=ancestor::model/stateMachine/@name)]
    [not(parameter[@name='automata']/@value=preceding::actionNode[@type='AUTOMATA_CALL']/parameter[@name='automata']/@value)]"/>


<!-- APPLY TEMPLATES FOR EACH ACTION NODE (SELECTED DISTINCTLY) -->
<xsl:template name="distinctActionNodes">
    <xsl:param name="mode"/>
    <xsl:call-template name="distinctNode">
        <xsl:with-param name="mode" select="$mode"/>
        <xsl:with-param name="nodeName" select="'actionNode'"/>
    </xsl:call-template>
</xsl:template>

<!-- APPLY TEMPLATES FOR EACH CONDITION NODE (SELECTED DISTINCTLY) -->
<xsl:template name="distinctConditionNodes">
    <xsl:param name="mode"/>
    <xsl:call-template name="distinctNode">
        <xsl:with-param name="mode" select="$mode"/>
        <xsl:with-param name="nodeName" select="'conditionNode'"/>
    </xsl:call-template>
</xsl:template>


<!-- TRANSITIONS -->
<xsl:template name="transitions">
    <xsl:param name="mode"/>
    <xsl:param name="parameter1" select="''"/>
    <xsl:param name="parameter2" select="''"/>
    <xsl:for-each select="ancestor::stateMachine/transition[current()/ancestor-or-self::state/@name = @sourceRef]">
        <xsl:sort select="count(condition)" data-type="number" order="descending"/>
        <xsl:sort select="string-length(@priority) = 0" data-type="text" order="ascending"/>
        <xsl:sort select="@priority" data-type="number"/>
        <xsl:sort select="@targetRef" data-type="text"/>

        <xsl:call-template name="callWithMode">
            <xsl:with-param name="mode" select="$mode"/>
            <xsl:with-param name="parameter1" select="$parameter1" />
            <xsl:with-param name="parameter2" select="$parameter2" />
        </xsl:call-template>

    </xsl:for-each>
</xsl:template>


<xsl:template name="distinctNode">
    <xsl:param name="mode"/>
    <xsl:param name="nodeName"/>
    <xsl:param name="parameter1" select="''"/>
    <xsl:param name="parameter2" select="''"/>
    <xsl:for-each select="//stateMachine//*[name()=$nodeName]">
        <xsl:sort select="ancestor::stateMachine/@name"/>
        <xsl:sort select="@type"/>
        <xsl:sort select="@name"/>

        <xsl:if test="count(preceding::*[name()=$nodeName][@type=current()/@type][@name = current()/@name][ancestor::stateMachine/@name = current()/ancestor::stateMachine/@name]) = 0">
            <xsl:call-template name="callWithMode">
                <xsl:with-param name="mode" select="$mode"/>
                <xsl:with-param name="parameter1" select="$parameter1" />
                <xsl:with-param name="parameter2" select="$parameter2" />
            </xsl:call-template>
        </xsl:if>
    </xsl:for-each>
</xsl:template>




<xsl:template name="callWithMode">
    <xsl:param name="mode"/>
    <xsl:param name="parameter1" select="''"/>
    <xsl:param name="parameter2" select="''"/>
        
    <xsl:choose>
        <xsl:when test="$mode='SWITCH_BLOCK'">
            <xsl:apply-templates select="." mode="SWITCH_BLOCK">
                <xsl:with-param name="parameter1" select="$parameter1"/>
                <xsl:with-param name="parameter2" select="$parameter2"/>
            </xsl:apply-templates>
        </xsl:when>
        <xsl:when test="$mode='FUNCTION_DECLARATION'">
            <xsl:apply-templates select="." mode="FUNCTION_DECLARATION">
                <xsl:with-param name="parameter1" select="$parameter1"/>
                <xsl:with-param name="parameter2" select="$parameter2"/>
            </xsl:apply-templates>
        </xsl:when>
        <xsl:when test="$mode='FUNCTION_DEFINITION'">
            <xsl:apply-templates select="." mode="FUNCTION_DEFINITION">
                <xsl:with-param name="parameter1" select="$parameter1"/>
                <xsl:with-param name="parameter2" select="$parameter2"/>
            </xsl:apply-templates>
        </xsl:when>
        <xsl:when test="$mode='STRING'">
            <xsl:apply-templates select="." mode="STRING">
                <xsl:with-param name="parameter1" select="$parameter1"/>
                <xsl:with-param name="parameter2" select="$parameter2"/>
            </xsl:apply-templates>
        </xsl:when>
        <xsl:otherwise>
            <xsl:apply-templates select=".">
                <xsl:with-param name="parameter1" select="$parameter1"/>
                <xsl:with-param name="parameter2" select="$parameter2"/>
            </xsl:apply-templates>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

</xsl:stylesheet>