<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method='text' indent="no"/>
<xsl:include href="MA_common.xslt"/> 

<xsl:template match="/model">

;--- this file is machine generated ---
;Model: <xsl:value-of select="@name"/>

INCLUDE mymacros.inc
.model small
.586

;PUBLIC declarations section=============================================
    <xsl:for-each select="stateMachine">
        public call<xsl:value-of select="@name"/>
    </xsl:for-each>

    <xsl:for-each select="$missedAutomatas">
        public call<xsl:value-of select="parameter[@name='automata']/@value"/>
    </xsl:for-each>

;END PUBLIC declarations section=========================================


.data
        MACRO_DATA
        ;string to output in case of concurrency error
        strBlock    db  "Concurrency error!!!$"

    <xsl:for-each select="stateMachine">
        y<xsl:value-of select="@name"/> dw 0        ;automata state
        block<xsl:value-of select="@name"/> dw 0    ;automata block variable (for concurrency error check)
        str<xsl:value-of select="@name"/> db "<xsl:value-of select="@name"/>", "$" ;name of the automata to pass to log function
    </xsl:for-each>

    <xsl:for-each select="$missedAutomatas">
        str<xsl:value-of select="parameter[@name='automata']/@value"/> db "<xsl:value-of select="parameter[@name='automata']/@value"/>", "$"
    </xsl:for-each>
    
    <xsl:call-template name="distinctActionNodes">
        <xsl:with-param name="mode" select="'STRING'"/>
    </xsl:call-template>
    
    <xsl:call-template name="distinctConditionNodes">
        <xsl:with-param name="mode" select="'STRING'"/>
    </xsl:call-template>
;END Strings to output===================================================


.code

;========================================================================
;Extern functions declaration
;========================================================================
    <xsl:call-template name="distinctActionNodes">
        <xsl:with-param name="mode" select="'FUNCTION_DEFINITION'"/>
    </xsl:call-template>
    
    <xsl:call-template name="distinctConditionNodes">
        <xsl:with-param name="mode" select="'FUNCTION_DEFINITION'"/>
    </xsl:call-template>

        EXTRN  logZ :near
        EXTRN  logX :near
        EXTRN  logA :near
        EXTRN  logAend :near
;========================================================================
;END Extern functions declaration
;========================================================================


<xsl:apply-templates select="stateMachine"></xsl:apply-templates>


;========================================================================
;WARNING!!! Automatas descriptions are missed!!!
;========================================================================
<xsl:for-each select="$missedAutomatas">
        call<xsl:value-of select="parameter[@name='automata']/@value"/> proc near
            mov ax, 0
            ret 0
        call<xsl:value-of select="parameter[@name='automata']/@value"/> endp
</xsl:for-each>
;========================================================================
;WARNING!!! Automatas descriptions are missed!!!
;========================================================================

end

</xsl:template>



<!--===========================================================================================-->
<!--Automata processing-->
<xsl:template match="stateMachine">
    ; &lt;summary&gt;
    ; <xsl:value-of select="@description" />
    ; &lt;/summary&gt;
    ; &lt;param name="event"&gt;event to process&lt;/param&gt; 
    call<xsl:value-of select="@name"/> proc

        PROC_START

        cmp     block<xsl:value-of select="@name"/>, 0
        je      PROCESS_<xsl:value-of select="@name"/>

        WRITE_STRING strBlock, ' '
        PROC_END
        mov     ax, 1
        ret     0

    PROCESS_<xsl:value-of select="@name"/>:
        mov     block<xsl:value-of select="@name"/>, 1
        mov     bx, -1
        xor     ax, ax
        mov     cx, [bp+4]  ;read event
        add     bp, 6

        mov     ax, y<xsl:value-of select="@name"/>
        push    ax
        push    cx
        PUSH_STRING str<xsl:value-of select="@name"/>
        call    logA
        POP_STRING
        add     sp, 4

        <xsl:apply-templates select="state//state[count(child::state) = 0]" mode="SWITCH_BLOCK">
            <xsl:sort select="@name" data-type="text" /> 
        </xsl:apply-templates>

    <!--Previous apply-templates will generate a reference to that point-->
    CASE_<xsl:call-template name="automataName"/>_<xsl:value-of select="count(state//state[count(child::state) = 0])"/>:

    END_CASE_BLOCK_<xsl:call-template name="automataName"/>:

        mov     ax, y<xsl:value-of select="@name"/>
        push    ax
        PUSH_STRING str<xsl:value-of select="@name"/>
        call    logAend
        POP_STRING
        add     sp, 2
     
        mov     block<xsl:value-of select="@name"/>, 0

        PROC_END
        mov ax, 0
        ret 0
    call<xsl:value-of select="@name"/> endp
</xsl:template>
<!--End Automata processing-->
<!--===========================================================================================-->


<xsl:template match="state//state" mode="SWITCH_BLOCK">
    CASE_<xsl:call-template name="automataName"/>_<xsl:value-of select="position()-1"/>:
        cmp y<xsl:value-of select="ancestor::stateMachine/@name"/>, <xsl:value-of select="@name"/>
        jne  CASE_<xsl:call-template name="automataName"/>_<xsl:value-of select="position()"/>

        ;process transitions
        <xsl:call-template name="transitions">
            <xsl:with-param name="mode" select="'SWITCH_BLOCK'"/>
            <xsl:with-param name="parameter1" select="position()"/>
        </xsl:call-template>
    jmp END_CASE_BLOCK_<xsl:call-template name="automataName"/>
</xsl:template>                                                      



<!--===========================================================================================-->
<!-- Transitions processing for creating switch block -->
<xsl:template match="transition[not(count(./condition) = 0)]" mode="SWITCH_BLOCK">
    <xsl:param name="parameter1"/>
    <xsl:variable name="OK">OK_<xsl:call-template name="automataName"/>_<xsl:value-of select="$parameter1"/>_<xsl:value-of select="count(preceding::transition)"/></xsl:variable>
    <xsl:variable name="FAIL">FAIL_<xsl:call-template name="automataName"/>_<xsl:value-of select="$parameter1"/>_<xsl:value-of select="count(preceding::transition)"/></xsl:variable>

    ;IF (   <xsl:apply-templates select="condition" mode="SWITCH_BLOCK">
                <xsl:with-param name="OK_point"><xsl:value-of select="$OK" /></xsl:with-param>
                <xsl:with-param name="FAIL_point"><xsl:value-of select="$FAIL" /></xsl:with-param>
            </xsl:apply-templates>
    ;) THEN BEGIN
        <xsl:value-of select="$OK" />:
                
            <xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK"/> 
            mov y<xsl:value-of select="ancestor::stateMachine/@name"/>, <xsl:value-of select="@targetRef" />
            jmp END_CASE_BLOCK_<xsl:call-template name="automataName"/>
    ;END
        <xsl:value-of select="$FAIL" />:
</xsl:template>

<xsl:template match="transition[count(./condition) = 0]" mode="SWITCH_BLOCK">
     ;IF (TRUE) BEGIN
         <xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK"/> 
         mov y<xsl:value-of select="ancestor::stateMachine/@name"/>, <xsl:value-of select="@targetRef" />
     ;END
</xsl:template>
<!-- End Transitions processing for creating switch block -->
<!--===========================================================================================-->

<xsl:template match="condition" mode="SWITCH_BLOCK">
    <xsl:param name="OK_point" select="OK_"/>
    <xsl:param name="FAIL_point" select="FAIL_"/>

    <xsl:apply-templates mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point"><xsl:value-of select="$OK_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point"><xsl:value-of select="$FAIL_point"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="outputAction" mode="SWITCH_BLOCK">
    <xsl:apply-templates mode="SWITCH_BLOCK"/>
</xsl:template>


<!--===========================================================================================-->
<!-- Action nodes -->
<xsl:template match="actionNode[@type='OUTPUT_ACTION']" mode="SWITCH_BLOCK">
            ;log output action
            PUSH_STRING str<xsl:value-of select="ancestor::stateMachine/@name"/>_z<xsl:value-of select="@name"/>
            call    logZ
            POP_STRING
            call <xsl:value-of select="ancestor::stateMachine/@name"/>_z<xsl:value-of select="@name"/>
</xsl:template>

<xsl:template match="actionNode[@type='AUTOMATA_CALL']" mode="SWITCH_BLOCK">
            mov     ax, <xsl:value-of select="parameter[@name='event']/@value"/>
            push    ax
            call    call<xsl:value-of select="parameter[@name='automata']/@value"/>
            pop     ax
</xsl:template>

<xsl:template match="actionNode" priority="0" mode="SWITCH_BLOCK">/* ERROR: Unknown action Node type: '<xsl:value-of select="@type"/>';*/</xsl:template>
<!-- End Action nodes -->
<!--===========================================================================================-->


<!--===========================================================================================-->
<!-- Condition nodes and operations -->
<xsl:template match="conditionNode[@type='INPUT_VARIABLE']" mode="SWITCH_BLOCK">
        <xsl:param name="OK_point" select="OK_"/>
        <xsl:param name="FAIL_point" select="FAIL_"/>

            call <xsl:value-of select="ancestor::stateMachine/@name"/>_x<xsl:value-of select="@name"/>
            mov     cx, ax
            push    cx
            PUSH_STRING str<xsl:value-of select="ancestor::stateMachine/@name"/>_x<xsl:value-of select="@name"/>
            call    logX
            POP_STRING
            add     sp, 2
            cmp     cx, 0
            je      <xsl:value-of select="$OK_point"/>
            jne     <xsl:value-of select="$FAIL_point"/>
</xsl:template>

<xsl:template match="conditionNode[@type='EVENT']" mode="SWITCH_BLOCK">
        <xsl:param name="OK_point" select="OK_"/>
        <xsl:param name="FAIL_point" select="FAIL_"/>

        <xsl:choose>
            <xsl:when test="contains(@name, 'X')">
                cmp     cx, <xsl:value-of select="translate(@name, 'X', '0')"/>
                jl      <xsl:value-of select="$FAIL_point"/>
                cmp     cx, <xsl:value-of select="translate(@name, 'X', '9')"/>
                jg      <xsl:value-of select="$FAIL_point"/>
                mov     bx, cx
                jmp     <xsl:value-of select="$OK_point"/>
            </xsl:when>
            <xsl:otherwise>
                cmp     cx, <xsl:value-of select="@name"/>
                je      <xsl:value-of select="$OK_point"/>
                jne     <xsl:value-of select="$FAIL_point"/>
            </xsl:otherwise>
        </xsl:choose>
</xsl:template>

<xsl:template match="conditionNode" priority="0" mode="SWITCH_BLOCK">; ERROR: Unknown action Node type: '<xsl:value-of select="@type"/>'; Please, correct the xslt file; </xsl:template>


<!--AND-->
<xsl:template match="binaryOperation[@type='AND']" mode="SWITCH_BLOCK">
    <xsl:param name="OK_point" select="OK_"/>
    <xsl:param name="FAIL_point" select="FAIL_"/>

    <xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point">and_<xsl:call-template name="automataName"/>_<xsl:value-of select="$OK_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point"><xsl:value-of select="$FAIL_point"/></xsl:with-param>
    </xsl:apply-templates>

    <xsl:text>
    
        </xsl:text>
        and_<xsl:call-template name="automataName"/>_<xsl:value-of select="$OK_point"/>:
    
    <xsl:apply-templates select="child::*[position()=2]" mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point"><xsl:value-of select="$OK_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point"><xsl:value-of select="$FAIL_point"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>


<!--OR-->
<xsl:template match="binaryOperation[@type='OR']" mode="SWITCH_BLOCK">
    <xsl:param name="OK_point" select="OK_"/>
    <xsl:param name="FAIL_point" select="FAIL_"/>

    <xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point"><xsl:value-of select="$OK_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point">or_<xsl:call-template name="automataName"/>_<xsl:value-of select="$FAIL_point"/></xsl:with-param>
    </xsl:apply-templates>

    <xsl:text>

        </xsl:text>
        or_<xsl:call-template name="automataName"/>_<xsl:value-of select="$FAIL_point"/>:
    
    <xsl:apply-templates select="child::*[position()=2]" mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point"><xsl:value-of select="$OK_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point"><xsl:value-of select="$FAIL_point"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>


<!--NOT-->
<xsl:template match="unaryOperation[@type='NOT']" mode="SWITCH_BLOCK">
    <xsl:param name="OK_point" select="OK_"/>
    <xsl:param name="FAIL_point" select="FAIL_"/>

    <xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK">
       <xsl:with-param name="OK_point"><xsl:value-of select="$FAIL_point"/></xsl:with-param>
       <xsl:with-param name="FAIL_point"><xsl:value-of select="$OK_point"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="binaryOperation" priority="0" mode="SWITCH_BLOCK">;Error: Unknown binary operation type: '<xsl:value-of select="@type"/>'; Please, correct the xslt file; </xsl:template>
<xsl:template match="unaryOperation" priority="0" mode="SWITCH_BLOCK">;Error: Unknown unary operation type: '<xsl:value-of select="@type"/>'; Please, correct the xslt file; </xsl:template>
<!-- End Condition nodes and operations -->
<!--===========================================================================================-->




<!--=======================================================================-->
<!--FUNCTION_DEFINITION-->
<!--=======================================================================-->
<xsl:template match="actionNode[@type='OUTPUT_ACTION']" mode="FUNCTION_DEFINITION">
        EXTRN  <xsl:value-of select="ancestor::stateMachine/@name"/>_z<xsl:value-of select="@name"/> :near
</xsl:template>

<xsl:template match="actionNode[@type='AUTOMATA_CALL']" mode="FUNCTION_DEFINITION" ></xsl:template>

<xsl:template match="actionNode" priority="0" mode="FUNCTION_DEFINITION" >
        ;ERROR: Unknown actionNode type: '<xsl:value-of select="@type"/>'; Please, correct xslt file;
</xsl:template>

<xsl:template match="conditionNode[@type='AUTOMATA_CALL']" mode="FUNCTION_DEFINITION"></xsl:template>

<xsl:template match="conditionNode[@type='INPUT_VARIABLE']" mode="FUNCTION_DEFINITION" >
        EXTRN  <xsl:value-of select="ancestor::stateMachine/@name"/>_x<xsl:value-of select="@name"/> :near
</xsl:template>

<xsl:template match="conditionNode[@type='EVENT']" mode="FUNCTION_DEFINITION" ></xsl:template>

<xsl:template match="conditionNode" priority="0" mode="FUNCTION_DEFINITION" >
        ;ERROR: Unknown conditionNode type: '<xsl:value-of select="@type"/>'; Please, correct xslt file;
</xsl:template>

<!--=======================================================================-->
<!--FUNCTION_DEFINITION END-->
<!--=======================================================================-->

<!--=======================================================================-->
<!--STRING-->
<!--=======================================================================-->
<xsl:template match="actionNode[@type='OUTPUT_ACTION']" mode="STRING">
        str<xsl:value-of select="ancestor::stateMachine/@name"/>_z<xsl:value-of select="@name"/> db "<xsl:value-of select="ancestor::stateMachine/@name"/>_z<xsl:value-of select="@name"/>", '$'
</xsl:template>

<xsl:template match="actionNode[@type='AUTOMATA_CALL']" mode="STRING" ></xsl:template>

<xsl:template match="actionNode" priority="0" mode="STRING" >
        ;ERROR: Unknown actionNode type: '<xsl:value-of select="@type"/>'; Please, correct xslt file;
</xsl:template>

<xsl:template match="conditionNode[@type='AUTOMATA_CALL']" mode="STRING"></xsl:template>

<xsl:template match="conditionNode[@type='INPUT_VARIABLE']" mode="STRING" >
        str<xsl:value-of select="ancestor::stateMachine/@name"/>_x<xsl:value-of select="@name"/> db "<xsl:value-of select="ancestor::stateMachine/@name"/>_x<xsl:value-of select="@name"/>", '$'
</xsl:template>

<xsl:template match="conditionNode[@type='EVENT']" mode="STRING" ></xsl:template>

<xsl:template match="conditionNode" priority="0" mode="STRING" >
        ;ERROR: Unknown conditionNode type: '<xsl:value-of select="@type"/>'; Please, correct xslt file;
</xsl:template>
<!--=======================================================================-->
<!--STRING END-->
<!--=======================================================================-->

</xsl:stylesheet>