INCLUDE mymacros.inc

.model small
.586
    
    public logZ
    public logX
    public logA
    public logAend

.data
    MACRO_DATA
    strZ_1  db, "Output variable $"

    strA_1  db, "Automata $"
    strA_2  db, "in state $"
    strA_3  db, "starts processing of the event $"

    strAend_1  db, "Automata $"
    strAend_2  db, "finishes processing in state $"

    strX_1  db, "Check input variable $"
    strX_2  db, "...TRUE$"
    strX_3  db, "...FALSE$"
    
    buf     db, '    ', ' $'
    space   db, ' $'

.code

    ; <summary>
    ; Log integer
    ; </summary>
    ; <param name="p1">Segment of the string, represents the output variable name</param> 
    ; <param name="p2">Offset of the string, represents the output variable name</param> 
    logZ proc near
      p1      equ [bp + 4]
      p2      equ [bp + 6]
        PROC_START

        ;Output variable z ...
        WRITE_STRING strZ_1

        ;... <output variable name> ...
        WRITE_STRING_EX p1, p2

        WRITE_STRING space, ' '

        PROC_END
        mov     ax, 0
        ret     0
    logZ endp

    ; <summary>
    ; Log integer
    ; </summary>
    ; <param name="p1">Segment of the string, represents the input variable name</param> 
    ; <param name="p2">Offset of the string, represents the input variable name</param> 
    ; <param name="p3">Input variable value</param> 
    logX proc near
      p1      equ [bp + 4]
      p2      equ [bp + 6]
      p3      equ [bp + 8]
        PROC_START

        ;Check input variable x ...
        WRITE_STRING strX_1

        ;... <input variable name> ...
        WRITE_STRING_EX p1, p2

        ;... TRUE/FALSE ...
        mov     ax, p3
        cmp     ax, 0
        je      TRUE
        WRITE_STRING strX_3
        jmp     FIN
      TRUE:
        WRITE_STRING strX_2
      FIN:
        
        WRITE_STRING space, ' '

        PROC_END
        mov     ax, 0   
        ret     0
    logX endp


    ; <summary>
    ; Log automata call
    ; </summary>
    ; <param name="p1">Segment of the string, represents the automata  name</param> 
    ; <param name="p2">Offset of the string, represents the automata name</param> 
    ; <param name="p3">Event number</param> 
    ; <param name="state">Current automata state</param> 
    logA proc near
      p1      equ [bp + 4]
      p2      equ [bp + 6]
      p3      equ [bp + 8]
      state   equ [bp + 10]
        PROC_START

        ;Automata ...
        WRITE_STRING strA_1

        ;... <input variable name> ...
        WRITE_STRING_EX p1, p2

        WRITE_STRING strA_2

        push    offset buf
        mov     ax, state
        push    ax
        call    DWordToStr
        add     sp, 4
        WRITE_STRING buf

        WRITE_STRING strA_3
        
        push    offset buf
        mov     ax, p3
        push    ax
        call    DWordToStr
        add     sp, 4
        WRITE_STRING buf

        
        WRITE_STRING space, ' '

        PROC_END
        mov     ax, 0   
        ret     0
    logA endp


    ; <summary>
    ; Log end of automata call
    ; </summary>
    ; <param name="p1">Segment of the string, represents the automata  name</param> 
    ; <param name="p2">Offset of the string, represents the automata name</param> 
    ; <param name="p3">Event number</param> 
    logAend proc near
      p1      equ [bp + 4]
      p2      equ [bp + 6]
      p3      equ [bp + 8]
        PROC_START

        ;Automata ...
        WRITE_STRING strAend_1

        ;... <input variable name> ...
        WRITE_STRING_EX p1, p2

        WRITE_STRING strAend_2

        push    offset buf
        mov     ax, p3
        push    ax
        call    DWordToStr
        add     sp, 4
        WRITE_STRING buf
        
        WRITE_STRING space, ' '

        PROC_END
        mov     ax, 0   
        ret     0
    logAend endp



;-------------------------------------------------------------------------------
; PROCEDURE : Fill
; PURPOSE   : Fill a block with given values
; PARAMS    : buffer - address of the block to fill
;             cnt    - number of bytes to write
;             value  - value to write
;-------------------------------------------------------------------------------
Fill PROC near
  buffer equ [bp + 4]
  cnt    equ [bp + 6]
  value  equ [bp + 8]
    PROC_START
    
    mov bx, buffer
    mov cx, cnt
    mov al, value
    dec bx
loop1:
    mov si, cx
    mov byte ptr [bx][si], al
    loop loop1
  
    PROC_END
    ret 0
Fill ENDP
;-------------------------------------------------------------------------------;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; PROCEDURE : DWordToStr
; PURPOSE   : Convert DWord to literal representation
; PARAMS    : number - DWord to convert
;             buff   - buffer to write to
;-------------------------------------------------------------------------------
DWordToStr PROC near
  value  equ [bp + 4]
  buffer equ [bp + 6]
    PROC_START
    
    mov ax, ' ' 
    push ax
    push 4
    mov ax, buffer
    push ax
    call Fill
    add sp, 6

    mov bx, buffer
    mov ax, value
    mov si, 4              ; si = intwidth - 1
    mov cx, 10             ; radix
    cmp ax, 0              ;
    jge rep0               ; if eax < 0 then
    mov byte ptr [bx], '-' ; put '-'
    neg ax                 ; change sign
rep0:
    cdq              ; prepare for idiv
    div cx           ; dl = x % 10
    add dl, '0'      ; dl = inttostr(dl)
    mov [bx][si], dl ; write a digit to the buffer
    dec si           ; inc the index
    cmp ax, 0h       ; if x div 10 > 0 then
    ja  rep0         ;   next decimal bit

    PROC_END
    ret 0                
DWordToStr ENDP    
;-------------------------------------------------------------------------------

end
