;-------------------------------------------------------------------------------
; PROJECT     : MetaAuto (assembler code generation)
; DESCRIPTION : Common macros
; AUTHOR      : Sergey Kanzhelev
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : MACRO_DATA
; DESCRIPTION : Data, used in following macros
MACRO_DATA MACRO
    crlf     db 0Dh,0Ah,'$'
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : COMMON_FIN
; DESCRIPTION : Common finalization
COMMON_FIN MACRO
  mov   ax, 4C00h
  int   21h
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : WRITE_STRING
; DESCRIPTION : Write a string
; PARAMS      : str - string to write
;               ln  - specify to use writeln
WRITE_STRING MACRO str, ln
  mov   dx, offset str
  mov   ah, 09h         
  int   21h             
  
  IFNB <ln>
    mov   dx, offset crlf
    mov   ah, 09h         
    int   21h             
  ENDIF
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : WRITE_STRING_EX
; DESCRIPTION : Write a string
; PARAMS      : strSegment - segment of the string to write
;               strOffset  - offset of the string to write
WRITE_STRING_EX MACRO strSegment, strOffset
  mov   ds, strSegment
  mov   dx, strOffset
  mov   ah, 09h         
  int   21h             
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : PROC_START
; DESCRIPTION : Move all registers to the stack
; PARAMS      : none
PROC_START MACRO
  push  bp
  mov   bp, sp
  pusha
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : PROC_END
; DESCRIPTION : Restore the registers values from the stack
; PARAMS      : none
PROC_END MACRO
  popa
  pop   bp
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : PUSH_STRING
; DESCRIPTION : Push string to the stack
; PARAMS      : none
PUSH_STRING MACRO str
  mov   ax, offset str
  push  ax
  mov   ax, seg str
  push  ax
ENDM
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; MACRO       : POP_STRING
; DESCRIPTION : Delete string from the stack
; PARAMS      : none
POP_STRING MACRO str
  add   sp, 4
ENDM
;-------------------------------------------------------------------------------
