<?xml version='1.0' ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method='text' indent="no" />
	<xsl:key name="distinctInputVariables" match="//conditionNode[@type='INPUT_VARIABLE']" use="parameter[@name='name']/@value" />
	<xsl:key name="distinctOutputActions" match="//actionNode[@type='OUTPUT_ACTION']" use="parameter[@name='name']/@value" />
	<xsl:key name="distinctAutomataCalls" match="//actionNode[@type='AUTOMATA_CALL']" use="parameter[@name='automata']/@value" />
	<xsl:template match="/model">
//--- this file is machine generated ---
//Model: <xsl:value-of select="@name" />

namespace Automatas
{
    public class BaseAutomata
    {
    }

<xsl:apply-templates select="stateMachine"></xsl:apply-templates>
}

</xsl:template>
	<!--===========================================================================================-->
	<!--Automata processing-->
	<xsl:template match="stateMachine">
    /// &lt;summary&gt;
    /// <xsl:value-of select="@description" />
    /// &lt;/summary&gt;
    public abstract class <xsl:value-of select="@name" /> : BaseAutomata
    {
        protected string y = "0";

        public void A(int e)
        {
            switch (y)
            {
                <xsl:apply-templates select="state//state[count(state) = 0]" mode="SWITCH_BLOCK">
					<xsl:sort select="@name" data-type="text" />
				</xsl:apply-templates>
            }
        }
	    <xsl:variable name="stateMachineName" select="@name" />
    	<xsl:apply-templates select="//actionNode[generate-id(.) = generate-id(key('distinctOutputActions', parameter[@name='name']/@value)[ancestor::stateMachine/@name=$stateMachineName])]" mode="FUNCTION_DEFINITIONS">
			<xsl:sort select="@type" />
			<xsl:sort select="@name" />
		</xsl:apply-templates>
    	<xsl:apply-templates select="//actionNode[generate-id(.) = generate-id(key('distinctAutomataCalls', parameter[@name='automata']/@value)[ancestor::stateMachine/@name=$stateMachineName])]" mode="FUNCTION_DEFINITIONS">
			<xsl:sort select="@type" />
			<xsl:sort select="@name" />
		</xsl:apply-templates>
    	<xsl:apply-templates select="//conditionNode[generate-id(.) = generate-id(key('distinctInputVariables', parameter[@name='name']/@value)[ancestor::stateMachine/@name=$stateMachineName])]" mode="FUNCTION_DEFINITIONS">
			<xsl:sort select="@type" />
			<xsl:sort select="@name" />
		</xsl:apply-templates>
    }
</xsl:template>
	<!--End Automata processing-->
	<!--===========================================================================================-->
	<xsl:template match="state//state" mode="SWITCH_BLOCK">
                case "<xsl:value-of select="@name" />":
				<xsl:apply-templates select="ancestor::stateMachine/transition[current()/ancestor-or-self::state/@name = @sourceRef]" mode="SWITCH_BLOCK">
			<xsl:sort select="count(condition)" data-type="number" order="descending" />
			<xsl:sort select="string-length(@priority) = 0" data-type="text" order="ascending" />
			<xsl:sort select="@priority" data-type="number" />
			<xsl:sort select="@targetRef" data-type="text" />
		</xsl:apply-templates>
                break;
</xsl:template>
	<!--===========================================================================================-->
	<!-- Transitions processing for creating switch block -->
	<xsl:template match="transition[not(count(./condition) = 0)]" mode="SWITCH_BLOCK">
		<xsl:choose>
			<xsl:when test="position() = 1">
                    if      (<xsl:apply-templates select="condition" mode="SWITCH_BLOCK" />) &#x9;&#x9; {<xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK" /> y = "<xsl:value-of select="@targetRef" />";}
        </xsl:when>
			<xsl:otherwise>
                    else if (<xsl:apply-templates select="condition" mode="SWITCH_BLOCK" />) &#x9;&#x9; {<xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK" /> y = "<xsl:value-of select="@targetRef" />";}
        </xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="transition[count(./condition) = 0]" mode="SWITCH_BLOCK">
		<xsl:choose>
			<xsl:when test="position() = 1">
                    if      (true) &#x9;&#x9; {<xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK" /> y = "<xsl:value-of select="@targetRef" />";}
        </xsl:when>
			<xsl:otherwise>
                    else if (true) &#x9;&#x9; {<xsl:apply-templates select="outputAction" mode="SWITCH_BLOCK" /> y = "<xsl:value-of select="@targetRef" />";}
        </xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- End Transitions processing for creating switch block -->
	<!--===========================================================================================-->
	<xsl:template match="condition" mode="SWITCH_BLOCK">
		<xsl:apply-templates mode="SWITCH_BLOCK" />
	</xsl:template>
	<xsl:template match="outputAction" mode="SWITCH_BLOCK">
		<xsl:apply-templates mode="SWITCH_BLOCK" />
	</xsl:template>
	<!--===========================================================================================-->
	<!-- Action nodes -->
	<xsl:template match="actionNode[@type='OUTPUT_ACTION']" mode="SWITCH_BLOCK">z<xsl:value-of select="@name" />(); </xsl:template>
	<xsl:template match="actionNode[@type='AUTOMATA_CALL']" mode="SWITCH_BLOCK">Call_<xsl:value-of select="parameter[@name='automata']/@value" />(<xsl:value-of select="parameter[@name='event']/@value" />); </xsl:template>
	<xsl:template match="actionNode" priority="0" mode="SWITCH_BLOCK">/* ERROR: Unknown action Node type: '<xsl:value-of select="@type" />';*/</xsl:template>
	<!-- End Action nodes -->
	<!--===========================================================================================-->
	<!--===========================================================================================-->
	<!-- Condition nodes and operations -->
	<xsl:template match="conditionNode[@type='INPUT_VARIABLE']" mode="SWITCH_BLOCK">x<xsl:value-of select="@name" />()</xsl:template>
	<xsl:template match="conditionNode[@type='EVENT']" mode="SWITCH_BLOCK">e == <xsl:value-of select="@name" /></xsl:template>
	<xsl:template match="conditionNode" priority="0" mode="SWITCH_BLOCK">/* ERROR: Unknown action Node type: '<xsl:value-of select="@type" />'; Please, correct the xslt file; */</xsl:template>
	<xsl:template match="binaryOperation[@type='AND']" mode="SWITCH_BLOCK">(<xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK" />) &amp;&amp; (<xsl:apply-templates select="child::*[position()=2]" mode="SWITCH_BLOCK" />)</xsl:template>
	<xsl:template match="binaryOperation[@type='OR']" mode="SWITCH_BLOCK">(<xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK" />) || (<xsl:apply-templates select="child::*[position()=2]" mode="SWITCH_BLOCK" />)</xsl:template>
	<xsl:template match="unaryOperation[@type='NOT']" mode="SWITCH_BLOCK">!(<xsl:apply-templates select="child::*[position()=1]" mode="SWITCH_BLOCK" />)</xsl:template>
	<xsl:template match="binaryOperation" priority="0" mode="SWITCH_BLOCK">/*Error: Unknown binary operation type: '<xsl:value-of select="@type" />'; Please, correct the xslt file; */</xsl:template>
	<xsl:template match="unaryOperation" priority="0" mode="SWITCH_BLOCK">/*Error: Unknown unary operation type: '<xsl:value-of select="@type" />'; Please, correct the xslt file; */</xsl:template>
	<!-- End Condition nodes and operations -->
	<!--===========================================================================================-->
	<xsl:template match="actionNode[@type='OUTPUT_ACTION']" mode="FUNCTION_DEFINITIONS">
		/// &lt;summary&gt;
		/// <xsl:value-of select="ancestor-or-self::stateMachine//node[(@type = current()/@type) and (@name = current()/@name)]/@description" />
		/// &lt;/summary&gt;
		protected abstract void z<xsl:value-of select="@name" />();
</xsl:template>
	<xsl:template match="actionNode[@type='AUTOMATA_CALL']" mode="FUNCTION_DEFINITIONS">
		/// &lt;summary&gt;
		/// Вызов реализации автомата <xsl:value-of select="parameter[@name='automata']/@value" />.
		/// &lt;/summary&gt;
		protected abstract void Call_<xsl:value-of select="parameter[@name='automata']/@value" />(int e);
</xsl:template>
	<xsl:template match="actionNode" priority="0" mode="FUNCTION_DEFINITIONS">
		/*ERROR: Unknown actionNode type: '<xsl:value-of select="@type" />'; Please, correct xslt file;*/
</xsl:template>
	<xsl:template match="conditionNode[@type='INPUT_VARIABLE']" mode="FUNCTION_DEFINITIONS">
		/// &lt;summary&gt;
		/// <xsl:value-of select="ancestor-or-self::stateMachine//node[(@type = current()/@type) and (@name = current()/@name)]/@description" />
		/// &lt;/summary&gt;
		/// &lt;returns&gt;True если условие выполнено, false - в противном случае&lt;/returns&gt;
		protected abstract bool x<xsl:value-of select="@name" />();
</xsl:template>
	<xsl:template match="conditionNode[@type='EVENT']" mode="FUNCTION_DEFINITIONS"></xsl:template>
	<xsl:template match="conditionNode" priority="0" mode="FUNCTION_DEFINITIONS">
		/*ERROR: Unknown conditionNode type: '<xsl:value-of select="@type" />'; Please, correct xslt file;*/
</xsl:template>
</xsl:stylesheet>