<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method='text' indent="no"/>


<xsl:template match="/model">
//--- this file is machine generated ---

#include "StdAfx.h"
#include "common.h"
#include "log.h"

common_t cm;

<xsl:for-each select="stateMachine">

//-------------------------------------------------------
// A<xsl:value-of select="@name"/> - <xsl:value-of select="@description" />
//-------------------------------------------------------
void A<xsl:value-of select="@name"/>( ubyte e )
{
  ubyte y_old = cm.y<xsl:value-of select="@name"/>;

  #ifdef A<xsl:value-of select="@name"/>_BEGIN_LOGGING
    log_a_begin(<xsl:value-of select="@name"/>, y_old, e);
  #endif

  switch( cm.y<xsl:value-of select="@name"/> )
  {

    <xsl:for-each select="//state[count(state) = 0]">
        <xsl:sort select="@name"/>
    case <xsl:value-of select="@name"/>: //<xsl:value-of select="description"/>_
      
      <xsl:for-each select="stateMachineRef/actionNode">
          A<xsl:value-of select="parameter[@name='automata']/@value"/>(e);
      </xsl:for-each>
      
	  <xsl:apply-templates select="ancestor::stateMachine/transition[current()/ancestor-or-self::state/@name = @sourceRef]">
        <xsl:sort select="count(condition)" data-type="number" order="descending"/>
        <xsl:sort select="string-length(@priority) = 0" data-type="text" order="ascending"/>
        <xsl:sort select="@priority" data-type="number"/>
        <xsl:sort select="@targetRef" data-type="text"/>
      </xsl:apply-templates>

    break;
    </xsl:for-each>

    default:
      #ifdef A<xsl:value-of select="@name"/>_ERRORS_LOGGING
        log_write(LOG_GRAPH_ERROR, "Unknown automata state!");
      #else
        ;
      #endif
  }

  if( y_old == cm.y<xsl:value-of select="@name"/> ) goto A<xsl:value-of select="@name"/>_end;

  #ifdef A<xsl:value-of select="@name"/>_TRANS_LOGGING
    log_a_trans(<xsl:value-of select="@name"/>, y_old, cm.y<xsl:value-of select="@name"/>);
  #endif

  switch( cm.y<xsl:value-of select="@name"/> )
  {
    <xsl:for-each select="//state[count(state) = 0]" >
    	<xsl:sort select="@name" />
    	case <xsl:value-of select="@name"/>: // <xsl:value-of select="description"/>
    		<xsl:apply-templates select="stateMachineRef/actionNode" />
    		<xsl:apply-templates select="outputAction/actionNode" />
    	break;
    </xsl:for-each>
  }

  A<xsl:value-of select="@name"/>_end: ;
  #ifdef A<xsl:value-of select="@name"/>_END_LOGGING
    log_a_end(<xsl:value-of select="@name"/>, cm.y<xsl:value-of select="@name"/>, e);
  #endif
}

</xsl:for-each>

<xsl:variable name="relatedAutomatas" select="//*[ ((name()='actionNode') and ((@type='AUTOMATA_CALL') or (@type='SIMPLE_AUTOMATA_CALL_IN_STATE'))) or
													((name()='conditionNode') and ((@type='OTHER_AUTOMATA_EVENT_NOTEQUAL') or (@type='OTHER_AUTOMATA_EVENT_EQUAL')))]
	[not(parameter[@name='automata']/@value=ancestor::model/stateMachine/@name)]
	[not(parameter[@name='automata']/@value=preceding::*[ ((name()='actionNode') and ((@type='AUTOMATA_CALL') or (@type='SIMPLE_AUTOMATA_CALL_IN_STATE'))) or
															((name()='conditionNode') and ((@type='OTHER_AUTOMATA_EVENT_NOTEQUAL') or (@type='OTHER_AUTOMATA_EVENT_EQUAL')))]/parameter[@name='automata']/@value)]" />

<xsl:for-each select="$relatedAutomatas" >
  void A<xsl:value-of select="parameter[@name='automata']/@value"/> ( ubyte e ) {}
</xsl:for-each>

</xsl:template>


<xsl:template match="transition">
    
  <xsl:choose>
    <xsl:when test="not(condition)">
   	  if (1)
   	</xsl:when>
   	<xsl:otherwise>
      if ( <xsl:apply-templates select="condition"/> )
   	</xsl:otherwise>
  </xsl:choose>
      {
        <xsl:apply-templates select="action/actionNode"/>
  	    cm.y<xsl:value-of select="ancestor::stateMachine/@name"/> = <xsl:value-of select="@targetRef"/>;
      }
  <xsl:if test="not(position()=last())">
  	  else
  </xsl:if>


</xsl:template>

<xsl:template match="actionNode[(@type='AUTOMATA_CALL') or (@type='SIMPLE_AUTOMATA_CALL_IN_STATE')]">
			//
            A<xsl:value-of select="parameter[@name='automata']/@value"/>(0);	
</xsl:template>

<xsl:template match="actionNode[@type='OUTPUT_ACTION']">
            z<xsl:value-of select="@name"/>();	
</xsl:template>


<xsl:template match="condition"><xsl:apply-templates select="*"/></xsl:template>


<xsl:template match="binaryOperation" priority="0">	( <xsl:apply-templates select="child::*[position()=1]"/> ) /*Unknown binary operation*/ ( <xsl:apply-templates select="child::*[position()=2]"/> ) </xsl:template>

<xsl:template match="unaryOperation" priority="0">	( /*Unknown unary operation*/ <xsl:apply-templates select="child::*[position()=1]"/> ) </xsl:template>

<xsl:template match="binaryOperation[@type='AND']" > ( <xsl:apply-templates select="child::*[position()=1]"/> ) &amp;&amp; ( <xsl:apply-templates select="child::*[position()=2]"/> ) </xsl:template>
<xsl:template match="binaryOperation[@type='OR']">  ( <xsl:apply-templates select="child::*[position()=1]"/> ) || ( <xsl:apply-templates select="child::*[position()=2]"/> ) </xsl:template>
<xsl:template match="unaryOperation[@type='NOT']"> !( <xsl:apply-templates select="child::*[position()=1]"/> ) </xsl:template>


<xsl:template 
	match="conditionNode[@type='OTHER_AUTOMATA_EVENT_EQUAL']"
	>cm.y<xsl:value-of select="parameter[@name='automata']/@value" /> == <xsl:value-of select="parameter[@name='event']/@value" />
</xsl:template>

<xsl:template 
	match="conditionNode[@type='OTHER_AUTOMATA_EVENT_NOTEQUAL']"
	>cm.y<xsl:value-of select="parameter[@name='automata']/@value" /> != <xsl:value-of select="parameter[@name='event']/@value" />
</xsl:template>

<xsl:template 
	match="conditionNode[@type='EVENT']"
	>e = <xsl:value-of select="@name" />
</xsl:template>

<xsl:template 
	match="conditionNode[@type='INPUT_VARIABLE']"
	>x<xsl:value-of select="@name" />()</xsl:template>

<xsl:template match="conditionNode" priority="0">/* Unknown condition node */</xsl:template>


</xsl:stylesheet>
