<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method='text' indent="no"/>

<xsl:key name="distinctEvents" match="//conditionNode[@type='EVENT']" use="parameter[@name='name']/@value"/>
<xsl:variable name="events" select="//conditionNode[generate-id(.) = generate-id(key('distinctEvents', parameter[@name='name']/@value))]" />
<xsl:key name="distinctInputVariables" match="//conditionNode[@type='INPUT_VARIABLE']" use="parameter[@name='name']/@value"/>
<xsl:variable name="inputVariables" select="//conditionNode[generate-id(.) = generate-id(key('distinctInputVariables', parameter[@name='name']/@value))]" />
<xsl:key name="distinctOutputActions" match="//actionNode[@type='OUTPUT_ACTION']" use="parameter[@name='name']/@value"/>
<xsl:variable name="outputVariables" select="//actionNode[generate-id(.) = generate-id(key('distinctOutputActions', parameter[@name='name']/@value))]" />


<xsl:template match="/model">

//--- this file is machine generated ---

#ifndef CommonH
#define CommonH

#include "types.h"

typedef struct{
 <xsl:for-each select="stateMachine">
  ubyte y<xsl:value-of select="@name"/>; // <xsl:value-of select="@description"/>
 </xsl:for-each>

<xsl:variable name="relatedAutomatas" select="//*[ ((name()='actionNode') and ((@type='AUTOMATA_CALL') or (@type='SIMPLE_AUTOMATA_CALL_IN_STATE'))) or
													((name()='conditionNode') and ((@type='OTHER_AUTOMATA_EVENT_NOTEQUAL') or (@type='OTHER_AUTOMATA_EVENT_EQUAL')))]
	[not(parameter[@name='automata']/@value=ancestor::model/stateMachine/@name)]
	[not(parameter[@name='automata']/@value=preceding::*[ ((name()='actionNode') and ((@type='AUTOMATA_CALL') or (@type='SIMPLE_AUTOMATA_CALL_IN_STATE'))) or
															((name()='conditionNode') and ((@type='OTHER_AUTOMATA_EVENT_NOTEQUAL') or (@type='OTHER_AUTOMATA_EVENT_EQUAL')))]/parameter[@name='automata']/@value)]" />

 <xsl:for-each select="$relatedAutomatas" >
  ubyte y<xsl:value-of select="parameter[@name='automata']/@value"/>;
 </xsl:for-each>

} common_t;

extern common_t cm;

// Automatas A
 <xsl:for-each select="stateMachine">
  void A<xsl:value-of select="@name"/>( ubyte e);
 </xsl:for-each>

 <xsl:for-each select="$relatedAutomatas" >
  void A<xsl:value-of select="parameter[@name='automata']/@value"/>( ubyte e ); 
 </xsl:for-each>

// Variables X
<xsl:for-each select="$inputVariables">
    <xsl:sort select="@name"/>
    ubyte x<xsl:value-of select="@name"/>(void); // <xsl:value-of select="ancestor::stateMachine/nodeDescriptions/node[@name=current()/@name][@type=current()/@type]/@description"/>
</xsl:for-each>


// Actions Z
<xsl:for-each select="$outputVariables">
    <xsl:sort select="@name"/>
    void z<xsl:value-of select="@name"/>(void); // <xsl:value-of select="ancestor::stateMachine/nodeDescriptions/node[@name=current()/@name][@type=current()/@type]/@description"/>
</xsl:for-each>

// Events E
<xsl:for-each select="$events">
    <xsl:sort select="@name"/>
    // e<xsl:value-of select="@name"/> - <xsl:value-of select="ancestor::stateMachine/nodeDescriptions/node[@name=current()/@name][@type=current()/@type]/@description"/>
</xsl:for-each>


#endif

</xsl:template>

</xsl:stylesheet>